package net;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SocketChannel;


public class Proxy implements Runnable {

    private SocketChannel peerChannel;

    private SocketChannel serverChannel;

    private String        remoteServer;

    private int           remoteServerPort;

    public Proxy(SocketChannel s, String remoteServer, int remoteServerPort) {
        this.peerChannel = s;
        this.remoteServer = remoteServer;
        this.remoteServerPort = remoteServerPort;
    }

    public void run() {
        try {
            serverChannel = SocketChannel.open(new InetSocketAddress(remoteServer, remoteServerPort));

            Listener.pool.execute(new ProxyAnswer(peerChannel, serverChannel));
            ByteBuffer buf = ByteBuffer.allocateDirect(1024);
            while (peerChannel.isOpen() && peerChannel.read(buf) != -1) {
                buf.flip();
                serverChannel.write(buf);
                buf.clear();
            }
        } catch (IOException e) {
            //e.printStackTrace();
        } finally {
            try {
                peerChannel.close();
                serverChannel.close();
            } catch (IOException e) {
                //e.printStackTrace();
            }
        }
    }


    private static class ProxyAnswer implements Runnable {

        private SocketChannel peer;

        private SocketChannel server;

        public ProxyAnswer(SocketChannel peerChannel, SocketChannel serverChannel) {
            this.peer = peerChannel;
            this.server = serverChannel;
        }

        public void run() {
            try {
                ByteBuffer buf = ByteBuffer.allocateDirect(1024);
                while (server.isOpen() && server.read(buf) != -1) {
                    buf.flip();
                    peer.write(buf);
                    buf.clear();
                }
            } catch (IOException e) {
                //e.printStackTrace();
            } finally {
                try {
                    peer.close();
                    server.close();
                } catch (IOException e) {
                    //e.printStackTrace();
                }
            }
        }

    }

}
